package crmdna.api.endpoint;

import com.google.api.server.spi.config.Api;
import com.google.appengine.api.users.User;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.gspreadsheet.GSpreadSheet;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;

@Api(name = "developersOnly")
public class GSpreadSheetApi {

    public APIResponse getPublishedSpreadsheetContents(@Named("spreadSheetKey") String gsKey,
                                                       @Nullable @Named("numLinesExclHeaderDefault2500") Integer numLinesExclHeader,
                                                       @Nullable @Named("showStackTraceIfErrorDefaultFalse") Boolean showStackTrace,
                                                       HttpServletRequest req) {

        try {
            if (numLinesExclHeader == null)
                numLinesExclHeader = 2500;

            List<Map<String, String>> listOfMap =
                    GSpreadSheet.getPublishedSpreadsheetAsListOfMap(gsKey, numLinesExclHeader);

            return new APIResponse().status(Status.SUCCESS).object(listOfMap);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().req(req));
        }
    }

    public APIResponse readSpreadsheetContents(@Nullable @Named("spreadSheetKey") String gsKey,
                                               HttpServletRequest req, User user) {

        try {

            // https://spreadsheets.google.com/feeds
            // https://www.googleapis.com/auth/drive

            if (gsKey == null) {
                gsKey = "1VCtKIPO71NpSN9mqwkt1Uki72clpKN_Qv_9VvBLyfZw";
            }

            String authString = req.getHeader("Authorization");
            if (authString == null) {
                return new APIResponse().status(Status.ERROR_LOGIN_REQUIRED);
            }

            String accessToken = authString.substring("Bearer ".length());

            return new APIResponse().status(Status.SUCCESS);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, true, new RequestInfo().req(req));
        }

    }
}
