package crmdna.api.endpoint;

import com.google.api.server.spi.config.Api;
import com.google.api.server.spi.config.ApiMethod;
import com.google.api.server.spi.config.ApiMethod.HttpMethod;
import com.google.appengine.api.users.User;
import crmdna.common.AssertUtils;
import crmdna.common.DateUtils;
import crmdna.common.StopWatch;
import crmdna.common.Utils;
import crmdna.common.ValidationResultProp;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.common.contact.Contact;
import crmdna.common.contact.ContactProp;
import crmdna.gspreadsheet.GSpreadSheet;
import crmdna.program.Program;
import crmdna.program.ProgramProp;
import crmdna.registration.*;
import crmdna.registration.Registration.RegistrationStatus;

import javax.annotation.Nullable;
import javax.inject.Named;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static crmdna.common.OfyService.ofy;

@Api(name = "registration", description = "Registration (Alpha)", clientIds = {
        "429804891913.apps.googleusercontent.com",
        com.google.api.server.spi.Constant.API_EXPLORER_CLIENT_ID})
public class RegistrationApi {
    @ApiMethod(path = "query", httpMethod = HttpMethod.GET)
    public APIResponse query(@Named("client") String client,
                             @Nullable @Named("programId") Long programId, @Nullable @Named("searchStr") String searchStr,
                             @Nullable @Named("status") RegistrationStatus status,
                             @Nullable @Named("sortByFirstName") Boolean sortByFirstName,
                             @Nullable @Named("showStackTrace") Boolean showStackTrace, HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        String login = null;

        try {
            RegistrationQueryCondition qc = new RegistrationQueryCondition();
            qc.programId = programId;
            qc.searchStr = searchStr;
            qc.status = status;

            if (sortByFirstName == null)
                sortByFirstName = false;
            qc.sortByFirstName = sortByFirstName;

            login = Utils.getLoginEmail(user);
            List<RegistrationProp> props = Registration.query(client, qc, login);

            return new APIResponse().status(Status.SUCCESS).object(props);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req)
                    .login(login));
        }
    }

    @ApiMethod(path = "queryByTransactionId", httpMethod = HttpMethod.GET)
    public APIResponse queryByTransactionId(@Named("client") String client,
                                            @Named("transactionId") String transactionId,
                                            @Nullable @Named("showStackTrace") Boolean showStackTrace, HttpServletRequest req) {

        try {
            if (null == client)
                client = "isha";

            RegistrationProp registrationProp = Registration.queryByTransactionId(client, transactionId);

            return new APIResponse().status(Status.SUCCESS).object(registrationProp);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "getByTransactionId", httpMethod = HttpMethod.GET)
    public APIResponse getByTransactionId(@Named("client") String client,
                                          @Named("transactionId") String transactionId,
                                          @Nullable @Named("showStackTrace") Boolean showStackTrace, HttpServletRequest req) {

        try {
            if (null == client)
                client = "isha";

            RegistrationProp registrationProp = Registration.getByTransactionId(client, transactionId);

            return new APIResponse().status(Status.SUCCESS).object(registrationProp);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "queryRegistrationStatus", httpMethod = HttpMethod.GET)
    public APIResponse queryRegistrationStatus(@Named("client") String client,
                                               @Named("programId") long programId, @Named("email") String email,
                                               @Named("firstName") String firstName, @Nullable @Named("lastName") String lastName,
                                               @Nullable @Named("showStackTrace") Boolean showStackTrace, HttpServletRequest req) {

        try {
            if (null == client)
                client = "isha";

            RegistrationStatusProp registrationStatusProp =
                    Registration.queryRegistrationStatus(client, programId, email, firstName, lastName);

            return new APIResponse().status(Status.SUCCESS).object(registrationStatusProp);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "getSummary", httpMethod = HttpMethod.GET)
    public APIResponse getSummary(@Named("client") String client,
                                  @Named("programId") long programId,
                                  @Nullable @Named("showStackTrace") Boolean showStackTrace, HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        String login = null;

        try {

            login = Utils.getLoginEmail(user);
            RegistrationSummaryProp registrationSummaryProp =
                    Registration.getSummary(client, programId, login);

            return new APIResponse().status(Status.SUCCESS).object(registrationSummaryProp);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "createDiscountCode", httpMethod = HttpMethod.GET)
    public APIResponse createDiscountCode(@Named("client") String client,
                                  @Named("programTypeIds") Set<Long> programTypeIds,
                                  @Named("discountCode") String discountCode,
                                  @Named("validTillYYYYMMDD") Integer validTillYYYYMMDD,
                                  @Nullable @Named("percentage") Double percentage,
                                  @Nullable @Named("amount") Double amount,
                                  @Nullable @Named("showStackTrace") Boolean showStackTrace,
                                  HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        String login = null;

        try {

            login = Utils.getLoginEmail(user);

            DiscountProp prop =
                    Discount.createDiscountCode(client, discountCode, programTypeIds, validTillYYYYMMDD, percentage, amount,
                        login);

            return new APIResponse().status(Status.SUCCESS).object(prop);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "updateDiscountCode", httpMethod = HttpMethod.GET)
    public APIResponse updateDiscountCode(@Named("client") String client,
                                          @Nullable @Named("programTypeIds") Set<Long> newProgramTypeIds,
                                          @Nullable @Named("discountCode") String newDiscountCode,
                                          @Nullable @Named("validTillYYYYMMDD") Integer newValidTillYYYYMMDD,
                                          @Nullable @Named("percentage") Double newPercentage,
                                          @Nullable @Named("amount") Double newAmount,
                                          @Nullable @Named("showStackTrace") Boolean showStackTrace,
                                          HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        String login = null;

        try {

            login = Utils.getLoginEmail(user);

            DiscountProp prop = Discount.updateDiscountCode(client, newDiscountCode, newProgramTypeIds, newValidTillYYYYMMDD,
                newPercentage, newAmount, login);

            return new APIResponse().status(Status.SUCCESS).object(prop);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "updateTimestamp", httpMethod = HttpMethod.POST)
    public APIResponse updateTimestamp(@Named("client") String client,
        @Named("programId") Long programId,
        @Named("emailId") String emailId,
        @Named("registrationStatus") RegistrationStatus rs,
        @Named("dateYYYYMMDD") Integer dateYYYYMMDD,
        @Nullable @Named("add") Boolean add,
        @Nullable @Named("showStackTrace") Boolean showStackTrace,
        HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        String login = null;

        try {

            login = Utils.getLoginEmail(user);

            RegistrationQueryCondition rqc = new RegistrationQueryCondition();
            rqc.programId = programId;
            rqc.email = emailId;

            RegistrationProp prop = Registration.query(client, rqc, login).get(0);
            AssertUtils.ensureNotNull(prop, "Registration not found");
            RegistrationEntity entity = Registration.safeGet(client, prop.registrationId);
            Date newDate = DateUtils.toDate(dateYYYYMMDD);

            if (add == null) add = false;

            int i = add ? prop.changes.size() : 0;

            for (; i < prop.changes.size(); i++) {
                RegistrationStatusChange rsc = prop.changes.get(i);
                if (rsc.newStatus == rs) {
                    entity.overrideStateChange(i, rs, newDate);
                    break;
                }
            }

            if (i == prop.changes.size()) {
                entity.overrideStateChange(i, rs, newDate);
            }

            ofy(client).save().entity(entity).now();

            return new APIResponse().status(Status.SUCCESS).object(entity.toProp());

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "cancel", httpMethod = HttpMethod.POST)
    public APIResponse cancel(@Named("client") String client,
        @Named("registrationIdOrEmail") String registrationIdOrEmail,
        @Nullable @Named("programId") Long programId,
        @Nullable @Named("showStackTrace") Boolean showStackTrace,
        HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        try {

            String login = Utils.getLoginEmail(user);

            RegistrationProp registrationProp = Registration.queryByIdOrEmail(client, registrationIdOrEmail, programId, login);
            registrationProp = Registration.cancel(client, registrationProp.registrationId, login);

            return new APIResponse().status(Status.SUCCESS).object(registrationProp);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "resendConfirmationEmail", httpMethod = HttpMethod.POST)
    public APIResponse resendConfirmationEmail(@Named("client") String client,
        @Named("registrationIdOrEmail") String registrationIdOrEmail,
        @Nullable @Named("programId") Long programId,
        @Nullable @Named("showStackTrace") Boolean showStackTrace,
        HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        try {
            String login = Utils.getLoginEmail(user);

            RegistrationProp registrationProp = Registration.queryByIdOrEmail(client, registrationIdOrEmail, programId, login);
            Registration.sendConfirmationEmail(client, Utils.getList(registrationProp), false); //resend forcefully

            return new APIResponse().status(Status.SUCCESS);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "resendConfirmationEmailsIfNotSent", httpMethod = HttpMethod.POST)
    public APIResponse resendConfirmationEmailsIfNotSent(@Named("client") String client,
        @Named("programId") long programId,
        @Nullable @Named("showStackTrace") Boolean showStackTrace,
        HttpServletRequest req, User user) {

        if (null == client)
            client = "isha";

        String login = null;

        try {

            login = Utils.getLoginEmail(user);

            ProgramProp programProp = Program.safeGet(client, programId).toProp(client);
            AssertUtils.ensureNotNull(
                Registration.getRegistrationMailContent(client, programProp),
                "RESERVED_REGISTRATION_CONFIRMATION MailContent missing");

            RegistrationQueryCondition qc = new RegistrationQueryCondition();
            qc.programId = programId;
            qc.status = RegistrationStatus.REGISTRATION_COMPLETE;
            List<RegistrationProp> registrationProps = Registration.query(client, qc, login);
            if (registrationProps.size() > 0) {
                Registration.sendConfirmationEmail(client, registrationProps, true);
                return new APIResponse().status(Status.SUCCESS);
            }

            return new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND);

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace, new RequestInfo().client(client).req(req));
        }
    }

    @ApiMethod(path = "uploadRegistrants", httpMethod = HttpMethod.POST)
    public APIResponse uploadRegistrants(@Named("client") String client,
        @Named("publishedSpreadSheetKey") String gsKey,
        @Named("programId") long programId,
        @Named("paymentReference") String paymentReference,
        @Named("amount") double amount,
        @Named("startIndex") int startIndex,
        @Named("numRecords") int numRecords,
        @Nullable @Named("ignoreWarningsDefaultFalse") Boolean ignoreWarnings,
        @Nullable @Named("showStackTrace") Boolean showStackTrace, HttpServletRequest req, User user) {

        String login = null;

        try {
            StopWatch sw = StopWatch.createStarted();

            login = Utils.getLoginEmail(user);
            List<Map<String, String>> listOfMap =
                GSpreadSheet.getPublishedSpreadsheetAsListOfMap(gsKey, numRecords, startIndex);

            List<ContactProp> contactDetailProps = Contact.getContactDetailsFromListOfMap(listOfMap);

            ValidationResultProp validationResultProp = Contact.validate(contactDetailProps, startIndex);
            if (validationResultProp.hasErrors()) {
                return new APIResponse()
                    .status(Status.ERROR_RESOURCE_INCORRECT)
                    .object(validationResultProp)
                    .message(
                        "Found [" + validationResultProp.getNumErrors() + "] errors and ["
                            + validationResultProp.getNumWarnings()
                            + "] warnings. Please correct them before uploading");
            }

            if (ignoreWarnings == null)
                ignoreWarnings = false;

            if (validationResultProp.hasWarnings() && !ignoreWarnings) {
                return new APIResponse()
                    .status(Status.ERROR_RESOURCE_INCORRECT)
                    .object(validationResultProp)
                    .message(
                        "Found [" + validationResultProp.getNumWarnings()
                            + "] warnings. To ignore warnings set [ignoreWarnings] flag to true");
            }

            List<RegistrationProp> registrationProps = new ArrayList<>();
            for (ContactProp contactProp : contactDetailProps) {
                contactProp.asOfyyyymmdd = DateUtils.toYYYYMMDD(new Date());
                RegistrationInput regInput = new RegistrationInput(contactProp, programId, amount, paymentReference);
                regInput.ignoreDateRestriction = true;
                registrationProps.add(Registration.register(client, regInput));
            }

            return new APIResponse().status(Status.SUCCESS)
                .processingTimeInMS(sw.msElapsed())
                .message("Registered [" + registrationProps.size() + "] participants");

        } catch (Exception ex) {
            return APIUtils.toAPIResponse(ex, showStackTrace,
                new RequestInfo().client(client).login(login).req(req));
        }
    }
}
