package crmdna.api.servlet;

import crmdna.calling.Campaign;
import crmdna.calling.CampaignEntity;
import crmdna.calling.CampaignProp;
import crmdna.calling.CampaignQueryCondition;
import crmdna.common.AssertUtils;
import crmdna.common.DateUtils;
import crmdna.common.DateUtils.DateRange;
import crmdna.common.Utils;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.group.Group;
import crmdna.interaction.Interaction;
import crmdna.interaction.Interaction.InteractionType;
import crmdna.interaction.InteractionProp;
import crmdna.interaction.InteractionQueryCondition;
import crmdna.interaction.InteractionQueryResult;
import crmdna.user.User;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class CampaignServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    private void query(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        CampaignQueryCondition qc = new CampaignQueryCondition();
        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;
        qc.groupIds.add(groupId);

        boolean updateStats = ServletUtils.getBoolParam(request, "getStats");

        List<CampaignEntity> campaignEntities = Campaign.query(client, qc, login);
        List<CampaignProp> props = new ArrayList<>();
        for (CampaignEntity entity : campaignEntities) {
            CampaignProp prop = entity.toProp();
            if (updateStats) Campaign.updateStats(client, prop);
            props.add(prop);
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(props));
    }

    private void getByUser(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        List<CampaignProp> campaignProps = Campaign.getCampaignsForUser(client,
            User.get(client, ServletUtils.getStrParam(request, "userEmail")).toProp(client).userId);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(campaignProps));
    }

    private void get(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        Long campaignId = ServletUtils.getLongParam(request, "campaignId");
        AssertUtils.ensureNotNull(campaignId);

        Map<Long, CampaignEntity> map = Campaign.get(client, Utils.getList(campaignId));
        AssertUtils.ensure(map.size() > 0, "Campaign " + campaignId + " not found");

        CampaignProp prop = map.get(campaignId).toProp();

        if (ServletUtils.getBoolParam(request, "getStats")) {
            Campaign.updateStats(client, prop);
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(prop));
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {
            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "query":
                        query(client, login, request, response);
                        break;
                    case "get":
                        get(client, login, request, response);
                        break;
                    case "getByUser":
                        getByUser(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }
}
