package crmdna.api.servlet;

import crmdna.calling.Campaign;
import crmdna.calling.CampaignEntity;
import crmdna.calling.CampaignProp;
import crmdna.common.AssertUtils;
import crmdna.common.DateUtils;
import crmdna.common.DateUtils.DateRange;
import crmdna.common.Utils;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.interaction.Interaction;
import crmdna.interaction.Interaction.InteractionType;
import crmdna.interaction.InteractionProp;
import crmdna.interaction.InteractionQueryCondition;
import crmdna.interaction.InteractionQueryResult;
import crmdna.user.User;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class InteractionServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    private void createInteraction(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        String strInteractionType = ServletUtils.getStrParam(request, "interactionType");
        InteractionType interactionType = (strInteractionType != null) ? InteractionType.valueOf(strInteractionType) : null;
        InteractionProp prop = Interaction.createInteraction(client, ServletUtils.getLongParam(request, "memberId"),
            ServletUtils.getStrParam(request, "content"), interactionType, new Date(), null, true, login);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(prop));
    }

    private void createSubInteraction(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        InteractionProp prop = Interaction.createSubInteraction(client, ServletUtils.getLongParam(request, "interactionId"),
            ServletUtils.getStrParam(request, "content"), new Date(), login);
        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).message("Added sub interaction").object(prop));
    }

    private void deleteInteraction(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        long interactionId = ServletUtils.getLongParam(request, "interactionId");
        Interaction.deleteInteraction(client, interactionId, login);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(
            "interaction Id [" + interactionId + "] deleted"));
    }

    private void deleteSubInteraction(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        Long interactionId = ServletUtils.getLongParam(request, "interactionId");
        Long subInteractionId = ServletUtils.getLongParam(request, "subInteractionId");
        Interaction.deleteSubInteraction(client, interactionId, subInteractionId, login);

        ServletUtils.setJson(response,
            new APIResponse().status(Status.SUCCESS)
                .message("Sub interaction [(" + interactionId + ") " + subInteractionId + "] deleted"));
    }

    private void updateInteraction(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        String strNewInteractionType = ServletUtils.getStrParam(request, "newInteractionType");
        InteractionType newInteractionType = (strNewInteractionType != null) ? InteractionType.valueOf(strNewInteractionType) : null;

        String strNewProgress = ServletUtils.getStrParam(request, "newProgress");
        Interaction.Progress newProgress = (strNewProgress != null) ? Interaction.Progress.valueOf(strNewProgress) : null;

        String strNewResponse = ServletUtils.getStrParam(request, "newResponse");
        Interaction.Response newResponse = (strNewResponse != null) ? Interaction.Response.valueOf(strNewResponse) : null;

        InteractionProp prop =
            Interaction.updateInteraction(client, ServletUtils.getLongParam(request, "interactionId"),
                ServletUtils.getLongParam(request, "newMemberId"), newInteractionType,
                ServletUtils.getStrParam(request, "newUserEmail"), newProgress, newResponse, login);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(prop));
    }

    private void updateSubInteraction(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        long subInteractionId = ServletUtils.getLongParam(request, "subInteractionId");
        Interaction.updateSubInteraction(client,
            ServletUtils.getLongParam(request, "interactionId"), subInteractionId,
            ServletUtils.getStrParam(request, "content"), new Date(), login);

        ServletUtils.setJson(response,
            new APIResponse().status(Status.SUCCESS).message("Sub interaction [" + subInteractionId + "] updated"));
    }

    private void query(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        String strInteractionType = ServletUtils.getStrParam(request, "interactionType");
        InteractionType interactionType = (strInteractionType != null) ? InteractionType.valueOf(strInteractionType) : null;

        String strDateRange = ServletUtils.getStrParam(request, "dateRange");
        DateRange dateRange = (strDateRange != null) ? DateRange.valueOf(strDateRange) : null;

        InteractionQueryCondition qc = new InteractionQueryCondition();
        Long memberId = ServletUtils.getLongParam(request, "memberId");
        if (memberId != null) {
            qc.memberIds.add(memberId);
        }
        Long userId = ServletUtils.getLongParam(request, "userId");
        if (userId != null) {
            qc.userIds.add(userId);
        }
        if (interactionType != null) {
            qc.interactionTypes.add(interactionType.toString());
        }
        qc.end = new Date();
        if (dateRange != null) {
            qc.start = new Date(qc.end.getTime() - DateUtils.getMilliSecondsFromDateRange(dateRange));
        }

        qc.campaignIds = ServletUtils.getLongParamsAsSet(request, "campaignId");

        String strProgress = ServletUtils.getStrParam(request, "progress");
        qc.progress = (strProgress != null) ? Interaction.Progress.valueOf(strProgress) : null;

        String strResponse = ServletUtils.getStrParam(request, "response");
        qc.response = (strResponse != null) ? Interaction.Response.valueOf(strResponse) : null;

        qc.startIndex = ServletUtils.getIntParam(request, "startIndex");
        qc.numResults = ServletUtils.getIntParam(request, "numResults");
        InteractionQueryResult result = Interaction.query(client, qc, login);

        if (ServletUtils.getBoolParam(request, "detailed")) {
            Interaction.populateDependents(client, result.interactionProps);
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(result));
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {
            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "createInteraction":
                        createInteraction(client, login, request, response);
                        break;
                    case "createSubInteraction":
                        createSubInteraction(client, login, request, response);
                        break;
                    case "deleteInteraction":
                        deleteInteraction(client, login, request, response);
                        break;
                    case "deleteSubInteraction":
                        deleteSubInteraction(client, login, request, response);
                        break;
                    case "updateInteraction":
                        updateInteraction(client, login, request, response);
                        break;
                    case "updateSubInteraction":
                        updateSubInteraction(client, login, request, response);
                        break;
                    case "query":
                        query(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }
}
