package crmdna.api.servlet;

import crmdna.client.Client;
import crmdna.common.api.APIException;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.encryption.Encryption;
import crmdna.member.MemberEntity;
import crmdna.member.MemberLoader;
import crmdna.member.MemberProp;
import crmdna.user.User;
import crmdna.user.UserProp;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.Arrays;
import java.util.logging.Logger;

public class LoginServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    private void get(String client, String login, HttpServletRequest request, HttpServletResponse response) throws Exception {

        User.ensureValidUser(ServletUtils.getStrParam(request, "client"), login);
        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(User.get(client, login).toProp(client)));
    }

    @SuppressWarnings("unused")
    private void login(String client, String login, HttpServletRequest request, HttpServletResponse response) throws Exception {

        String email = request.getParameter("email");
        String password = request.getParameter("password");

        APIResponse apiResponse;

        try {
            MemberEntity memberEntity = MemberLoader.getByEmail(client, email);
            if (memberEntity == null) {
                throw new APIException().status(Status.ERROR_INVALID_USER).message("Invalid User");
            }

            MemberProp memberProp = memberEntity.toProp();
            if (!memberProp.hasAccount) {
                throw new APIException().status(Status.ERROR_INVALID_USER).message("Invalid User");
            }

            byte[] encryptedPwd = Encryption.getEncryptedPassword(password, memberProp.getSalt());
            if (!Arrays.equals(memberProp.getEncryptedPwd(), encryptedPwd)) {
                throw new APIException().status(Status.ERROR_AUTH_FAILURE).message("Invalid Password");
            }

            if (!memberProp.isEmailVerified) {
                throw new APIException().status(Status.ERROR_PRECONDITION_FAILED).message(
                    "User pending verification");
            }

            HttpSession sess = request.getSession(true);
            sess.setAttribute("login", email);
            sess.setAttribute("loginType", "normal");

            apiResponse = new APIResponse().status(Status.SUCCESS).object(email);

        } catch (Exception e) {
            apiResponse =
                APIUtils.toAPIResponse(e, true, new RequestInfo().client(client).req(request));
        }

        String successUrl = request.getParameter("successUrl");

        if (successUrl != null) {
            String errorUrl = request.getParameter("errorUrl");
            response.sendRedirect((apiResponse.statusCode == Status.SUCCESS ? successUrl : errorUrl)
                + "?status=" + apiResponse.statusCode + "&message=" + apiResponse.userFriendlyMessage);
        } else {
            ServletUtils.setJson(response, apiResponse);
        }
    }

    @SuppressWarnings("unused")
    private void googleLogin(String client, String login, HttpServletRequest request, HttpServletResponse response) throws Exception {

        String redirectUri = ServletUtils.getRedirectUri(request);
        String onSuccessUrl = request.getParameter("onSuccessUrl");
        String onErrorUrl = request.getParameter("onErrorUrl");

        response.sendRedirect(
            ServletUtils.newFlow()
                .newAuthorizationUrl()
                .setState(onSuccessUrl + "|" + onErrorUrl + "|" + client)
                .setRedirectUri(redirectUri)
                .build());
    }

    @SuppressWarnings("unused")
    private void adminLogin(String client, String login, HttpServletRequest request, HttpServletResponse response) throws Exception {

        String email = request.getParameter("email");
        String[] passwords = request.getParameter("password").split(":");
        String user = passwords[0];
        String password = passwords[1];

        User.ensureValidUser(client, email);
        UserProp userProp = User.get(client, email).toProp(client);

        Logger logger = Logger.getLogger("admin-login");
        logger.info("email " + email);
        logger.info("user " + user);
        logger.info("password " + password);

        String adminPassword = Client.getAdminPassword(client, email);

        logger.info("adminPassword " + adminPassword);

        if (Client.getAdminPassword(client, email).equals(password)) {
            HttpSession sess = request.getSession(true);
            sess.setAttribute("login", user);
            sess.setAttribute("loginType", "normal");
        }

        response.sendRedirect(request.getParameter("onSuccessUrl"));
    }

    @SuppressWarnings("unused")
    private void logout(String client, String login, HttpServletRequest request, HttpServletResponse response) throws Exception {

        request.getSession(true).invalidate();
        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils
                .setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {

            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "get":
                        get(client, login, request, response);
                        break;
                    case "login":
                        login(client, login, request, response);
                        break;
                    case "google-login":
                        googleLogin(client, login, request, response);
                        break;
                    case "admin-login":
                        adminLogin(client, login, request, response);
                        break;
                    case "logout":
                        logout(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }
}
