package crmdna.api.servlet;

import com.google.api.client.repackaged.com.google.common.base.Strings;
import crmdna.common.DateUtils;
import crmdna.common.Utils;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.group.Group;
import crmdna.list.ListProp;
import crmdna.mail2.*;
import crmdna.member.MemberQueryCondition;
import crmdna.member.MemberUtils;
import crmdna.registration.Registration;
import crmdna.user.User;
import crmdna.user.UserProp;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

public class MailServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    private MailContentProp sanitize(String client, HttpServletRequest request, MailContentProp mailContentProp) {
        mailContentProp.bodyUrl =
            request.getScheme() + "://" + request.getServerName() + ":" + request.getServerPort() + "/mailContent" + "?client="
                + client + "&mailContentId=" + mailContentProp.mailContentId + "&action=viewcontent";
        mailContentProp.body = null;
        return mailContentProp;
    }

    private void createMailContent(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String bodyUrl = ServletUtils.getStrParam(request, "bodyUrl");
        String displayName = ServletUtils.getStrParam(request, "displayName");
        String group = ServletUtils.getStrParam(request, "group");
        String subject = ServletUtils.getStrParam(request, "subject");

        long groupId = Group.safeGetByIdOrName(client, group).toProp().groupId;
        String bodyHtml = Utils.readDataFromURL(bodyUrl);

        MailContentProp mailContentProp = MailContent.create(client, displayName, groupId, subject, bodyHtml, login);
        ServletUtils
            .setJson(response, new APIResponse().status(Status.SUCCESS).object(sanitize(client, request, mailContentProp)));
    }

    private void queryMailContent(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        Integer startYYYYMMDD = ServletUtils.getIntParam(request, "startYYYYMMDD");
        Integer endYYYYMMDD = ServletUtils.getIntParam(request, "endYYYYMMDD");

        Long startMS = null;
        Long endMS = null;

        if (startYYYYMMDD != null) {
            startMS = DateUtils.toDate(startYYYYMMDD).getTime();
        }

        if (endYYYYMMDD != null) {
            endMS = DateUtils.toDate(endYYYYMMDD).getTime();
        }

        Set<String> tags = ServletUtils.getStringParamsAsSet(request, "tag");

        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;

        List<MailContentEntity> entities = MailContent.query(client, groupId, null, startMS, endMS, tags, login);

        List<MailContentProp> props = new ArrayList<>();
        for (MailContentEntity entity : entities) {
            props.add(sanitize(client, request, entity.toProp()));
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(props));
    }

    private void getMailContent(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        MailContentEntity entity = MailContent.safeGet(client, ServletUtils.getLongParam(request, "mailContentId"));
        ServletUtils
            .setJson(response, new APIResponse().status(Status.SUCCESS).object(sanitize(client, request, entity.toProp())));
    }

    private void updateMailContent(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        long mailContentId = ServletUtils.getLongParam(request, "mailContentId");
        String newBodyUrl = ServletUtils.getStrParam(request, "newBodyUrl");
        String newDisplayName = ServletUtils.getStrParam(request, "newDisplayName");
        String newSubject = ServletUtils.getStrParam(request, "newSubject");
        Boolean allowUpdateIfMailsSent = ServletUtils.getBoolParam(request, "allowUpdateIfMailsSent");

        String newBodyHtml = Utils.readDataFromURL(newBodyUrl);

        MailContentProp mailContentProp =
            MailContent.update(client, mailContentId, newDisplayName, newSubject, newBodyHtml, allowUpdateIfMailsSent, login);
        ServletUtils
            .setJson(response, new APIResponse().status(Status.SUCCESS).object(sanitize(client, request, mailContentProp)));
    }

    private void sendEmailToMembers(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws Exception {

        long mailContentId = ServletUtils.getLongParam(request, "mailContentId");
        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;
        String sender = ServletUtils.getStrParam(request, "sender");
        MemberQueryCondition qc = MemberUtils.getQueryCondition(client, request);

        List<SentMailEntity> sentMailEntities =
            Mail.sendToMembers(client, groupId, qc, mailContentId, sender, null, login, "Friend", "Friend");

        List<SentMailProp> sentMailProps = new ArrayList<>();
        for (SentMailEntity sentMailEntity : sentMailEntities) {
            sentMailProps.add(sentMailEntity.toProp());
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(sentMailProps.size()));
    }

    private void sendToLoggedInUser(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws Exception {

        UserProp userProp = User.safeGet(client, login).toProp(client);
        MailMap mailMap = new MailMap();
        mailMap.add(login,
            Strings.isNullOrEmpty(userProp.firstName) ? "FirstName" : userProp.firstName,
            Strings.isNullOrEmpty(userProp.lastName) ? "LastName" : userProp.lastName);

        MailSendInput msi = new MailSendInput();
        msi.createMember = false;
        msi.groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;
        msi.isTransactionEmail = false;
        msi.mailContentId = ServletUtils.getLongParam(request, "mailContentId");
        msi.senderEmail = ServletUtils.getStrParam(request, "sender");
        msi.suppressIfAlreadySent = false;

        List<SentMailEntity> sentMailEntities = Mail.send(client, msi, mailMap, login);

        List<SentMailProp> sentMailProps = new ArrayList<>();
        for (SentMailEntity sentMailEntity : sentMailEntities) {
            sentMailProps.add(sentMailEntity.toProp());
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(sentMailProps));
    }

    private void getLists(String client, String login, HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;

        List<ListProp> props = crmdna.list.List.querySortedProps(client, groupId);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(props));
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {

            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "createMailContent":
                        createMailContent(client, login, request, response);
                        break;
                    case "queryMailContent":
                        queryMailContent(client, login, request, response);
                        break;
                    case "getMailContent":
                        getMailContent(client, login, request, response);
                        break;
                    case "updateMailContent":
                        updateMailContent(client, login, request, response);
                        break;
                    case "sendEmailToMembers":
                        sendEmailToMembers(client, login, request, response);
                        break;
                    case "sendToLoggedInUser":
                        sendToLoggedInUser(client, login, request, response);
                        break;
                    case "getLists":
                        getLists(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response,
                    APIUtils.toAPIResponse(ex, true, new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }
}
