package crmdna.api.servlet;

import crmdna.common.DateUtils;
import crmdna.common.NumberUtils;
import crmdna.common.Utils;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.group.Group;
import crmdna.mail2.MailContent;
import crmdna.mail2.MailContentEntity;
import crmdna.mail2.MailContentProp;
import crmdna.payment.Receipt;
import crmdna.payment.Receipt.ReceiptQueryCondition;
import crmdna.payment.ReceiptProp;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.List;

import static crmdna.common.AssertUtils.ensureNotNull;

@SuppressWarnings("serial")
public class ReceiptServlet extends HttpServlet {

    private void generate(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        String type = request.getParameter("type");
        ReceiptProp receiptProp = null;

        if (type.equals("adhoc")) {
            receiptProp = Receipt.generateForAdhocDonation(client,
                    Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId,
                    ServletUtils.getStrParam(request, "firstName"),
                    ServletUtils.getStrParam(request, "lastName"),
                    ServletUtils.getStrParam(request, "email"),
                    ServletUtils.getStrParam(request, "purpose"),
                    Utils.Currency.valueOf(ServletUtils.getStrParam(request, "ccy")),
                    ServletUtils.getDoubleParam(request, "amount"),
                    ServletUtils.getIntParam(request, "dateYYYYMMDD"),
                    true);
        }

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(receiptProp));
    }

    private void view(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        response.getWriter().println(
            Receipt.getBody(client,
                Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId,
                ServletUtils.getStrParam(request, "id")));
    }

    private void query(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        ReceiptQueryCondition rqc = new ReceiptQueryCondition();
        rqc.startYYYYMMDD = ServletUtils.getIntParam(request, "startYYYYMMDD");
        rqc.endYYYYMMDD = ServletUtils.getIntParam(request, "endYYYYMMDD");
        rqc.groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;

        List<ReceiptProp> receiptProps = Receipt.query(client, rqc);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(receiptProps));
    }

    private void resendEmail(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        ReceiptProp receiptProp =
            Receipt.safeGet(client, ServletUtils.getStrParam(request, "id")).toProp(client);
        Receipt.sendEmail(client, receiptProp);
        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException {

        String action = request.getParameter("action");
        String client = request.getParameter("client");
        String login = ServletUtils.getLogin(request);

        try {
            switch (action) {
                case "generate":
                    generate(client, login, request, response);
                    break;
                case "query":
                    query(client, login, request, response);
                    break;
                case "view":
                    view(client, login, request, response);
                    break;
                case "resendEmail":
                    resendEmail(client, login, request, response);
                    break;
                default:
                    ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
            }
        } catch (Exception ex) {
            ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
        }
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        doPost(request, response);
    }
}
