package crmdna.api.servlet;

import crmdna.client.isha.IshaConfig;
import crmdna.client.isha.IshaRegistration;
import crmdna.common.Utils;
import crmdna.common.api.APIException;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.member.MemberLoader;
import crmdna.member.MemberQueryCondition;
import crmdna.program.Program;
import crmdna.program.ProgramProp;
import crmdna.registration.Discount;
import crmdna.registration.DiscountProp;
import crmdna.registration.Registration;
import crmdna.registration.RegistrationInput;
import crmdna.registration.RegistrationProp;
import crmdna.registration.RegistrationQueryCondition;
import crmdna.registration.RegistrationSummaryProp;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.logging.Logger;

@SuppressWarnings("serial")
public class RegistrationServlet extends HttpServlet {

    public static RegistrationProp registerForProgram(HttpServletRequest req, HttpServletResponse resp)
            throws IOException {

        String client = null;
        RegistrationProp registrationProp = null;
        boolean offlineReg = ServletUtils.getBoolParam(req, "offline");

        try {

            client = ServletUtils.getStrParam(req, "client");
            Logger logger = Logger.getLogger(RegistrationServlet.class.getName());

            RegistrationInput regInput = new RegistrationInput();
            regInput.firstName = ServletUtils.getStrParam(req, "firstName");
            regInput.lastName = ServletUtils.getStrParam(req, "lastName");
            regInput.nickName = ServletUtils.getStrParam(req, "nickName");
            regInput.gender = ServletUtils.getStrParam(req, "gender");
            regInput.email = ServletUtils.getStrParam(req, "email");
            regInput.mobilePhone = ServletUtils.getStrParam(req, "mobilePhone");
            regInput.homePhone = ServletUtils.getStrParam(req, "homePhone");
            regInput.officePhone = ServletUtils.getStrParam(req, "officePhone");
            regInput.country = ServletUtils.getStrParam(req, "country");
            regInput.postalCode = ServletUtils.getStrParam(req, "postalCode");
            regInput.programId = ServletUtils.getLongParam(req, "programId");
            regInput.fee = ServletUtils.getDoubleParam(req, "fee");
            regInput.batchNo = ServletUtils.getIntParam(req, "batchNo");
            regInput.marketingChannel = ServletUtils.getStrParam(req, "marketingChannel");
            regInput.successCallbackUrl = ServletUtils.getStrParam(req, "successUrl");
            regInput.errorCallbackUrl = ServletUtils.getStrParam(req, "errorUrl");
            regInput.rootUrl = req.getRequestURL().toString();
            regInput.sendEmail = true;

            if (offlineReg) {
                regInput.sendEmail = false;
            } else {
                Utils.ensureNotNullOrEmpty(regInput.successCallbackUrl, "successUrl missing");
            }

            registrationProp = Registration.register(client, regInput);

            if (offlineReg) {
                ServletUtils.setJson(resp, new APIResponse().status(Status.SUCCESS).object(registrationProp));
                return registrationProp;
            }

            switch (registrationProp.getStatus()) {
                case REGISTRATION_COMPLETE:
                    Map<String, Object> map = new TreeMap<>();

                    map.put("email", registrationProp.email);
                    map.put("timestamp", registrationProp.getStatusDate());
                    map.put("firstName", registrationProp.firstName);
                    map.put("registrationId", registrationProp.registrationId + "");
                    map.put("programId", registrationProp.programId + "");

                    if (registrationProp.lastName != null)
                        map.put("lastName", registrationProp.lastName);

                    if (registrationProp.transactionId != null)
                        map.put("transactionId", registrationProp.transactionId);

                    map.put("status", registrationProp.getStatus());

                    ProgramProp programProp = Program.safeGet(client, registrationProp.programId).toProp(client);

                    String programName = programProp.programTypeProp.displayName;
                    if (programProp.description != null) programName += "-" + programProp.description;
                    map.put("programName", programName);

                    String redirectUrl;
                    if (registrationProp.alreadyRegistered) {
                        map.put("alreadyRegistered", true);
                        redirectUrl = Utils.getUrl(registrationProp.errorCallbackUrl, map);
                    } else {
                        redirectUrl = Utils.getUrl(registrationProp.successCallbackUrl, map);
                    }

                    resp.setStatus(HttpServletResponse.SC_SEE_OTHER);
                    Utils.ensureValidUrl(redirectUrl);

                    logger.info("redirect URL [" + redirectUrl + "]");
                    resp.setHeader("Location", redirectUrl);
                    break;

                case PAYMENT_AUTHORIZATION_PENDING:
                    resp.setStatus(HttpServletResponse.SC_SEE_OTHER);
                    Utils.ensureValidUrl(registrationProp.paymentUrl);
                    resp.setHeader("Location", registrationProp.paymentUrl);
                    break;

                default:
                    // should never come here
                    throw new APIException().status(Status.ERROR_INTERNAL).message(
                            "Invalid registration status [" + registrationProp.getStatus()
                                    + "] in RegistrationServlet after call to register.");
            }
        } catch (Exception ex) {
            APIResponse apiResponse = APIUtils.toAPIResponse(ex, true, new RequestInfo().client(client).req(req));

            if (offlineReg) {
                ServletUtils.setJson(resp, apiResponse);
            } else {
                String errMessage =
                    "An error occurred. Please try again.\n\n" + "Error code: " + apiResponse.statusCode
                        + "\nMessage: " + apiResponse.userFriendlyMessage + "\n\nStack trace: "
                        + apiResponse.object;
                Logger logger = Logger.getLogger(RegistrationServlet.class.getName());
                logger.warning(errMessage);
                resp.getWriter().println(errMessage);
            }
        }

        return registrationProp;
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {

            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                if (action.equals("getSummary")) {
                    long programId = ServletUtils.getLongParam(request, "programId");
                    RegistrationSummaryProp rsp = Registration.getSummary(client, programId, login);

                    MemberQueryCondition mqc = new MemberQueryCondition(client, 10000);
                    mqc.programIds.add(programId);
                    rsp.numParticipants = MemberLoader.getCount(mqc, login);

                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(rsp));
                } else if (action.equals("query")) {
                    RegistrationQueryCondition qc = new RegistrationQueryCondition();
                    qc.programId = ServletUtils.getLongParam(request, "programId");
                    qc.searchStr = ServletUtils.getStrParam(request, "searchStr");
                    List<RegistrationProp> props = Registration.query(client, qc, login);

                    if (client.equals(IshaConfig.getClient())) {
                        IshaRegistration.updateQueryProps(props);
                    }

                    if (ServletUtils.getBoolParam(request, "updateCheckinStatus")) {
                        Registration.updateCheckinStatus(client, props);
                    }

                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(props));

                } else if (action.equals("transfer")) {

                    Registration.transfer(client, ServletUtils.getLongParam(request, "registrationId"),
                            ServletUtils.getLongParam(request, "programId"), login);

                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));

                } else if (action.equals("overrideAsCompleted")) {

                    Registration.overrideAsCompleted(client, ServletUtils.getLongParam(request, "registrationId"), login);
                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));

                } else if (action.equals("resendConfirmationEmail")) {

                    RegistrationProp registrationProp =
                        Registration.safeGet(client, ServletUtils.getLongParam(request, "registrationId")).toProp();
                    Registration.sendConfirmationEmail(client, Utils.getList(registrationProp), false);
                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));

                } else if (action.equals("invalidate")) {

                    Registration.invalidate(client, ServletUtils.getLongParam(request, "registrationId"),
                            login);

                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));

                } else if (action.equals("cancel")) {

                    Registration.cancel(client, ServletUtils.getLongParam(request, "registrationId"), login);

                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS));

                } else if (action.equals("registerForProgram")) {

                    registerForProgram(request, response);

                } else if (action.equals("applyDiscount")) {

                    DiscountProp prop = Discount.applyDiscount(client,
                            ServletUtils.getStrParam(request, "discountCode"),
                            ServletUtils.getLongParam(request, "programId"));

                    ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(prop));
                } else {
                    ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(
                        response,
                        APIUtils.toAPIResponse(ex, true,
                                new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }
}
