package crmdna.api.servlet;

import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.group.Group;
import crmdna.member.MemberLoader;
import crmdna.member.MemberProp;
import crmdna.user.User;
import crmdna.user.UserProp;
import crmdna.venue.Venue;
import crmdna.venue.Venue.VenueProp;

import javax.servlet.Servlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Set;

public class UserServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    private void create(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws IOException {

        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;
        UserProp userProp = User.create(client, ServletUtils.getStrParam(request, "email"), groupId, login);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(userProp));
    }

    private void linkMember(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws IOException {

        UserProp userProp =
            User.linkMember(client, ServletUtils.getStrParam(request, "email"), ServletUtils.getLongParam(request, "memberId"), login);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS).object(userProp));
    }

    private void getAll(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId;
        List<UserProp> userProps = User.getAllForGroup(client, groupId, login);

        ServletUtils.setJson(response, new APIResponse().object(userProps).status(Status.SUCCESS));
    }

    private void getClientLevelPrivileges(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        ServletUtils.setJson(response, new APIResponse().object(User.ClientLevelPrivilege.values()).status(Status.SUCCESS));
    }

    private void getGroupLevelPrivileges(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        ServletUtils.setJson(response, new APIResponse().object(User.GroupLevelPrivilege.values()).status(Status.SUCCESS));
    }

    private void updateClientLevelPrivileges(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        String email = ServletUtils.getStrParam(request, "email");

        Set<String> privilegesToBeAdded = ServletUtils.getStringParamsAsSet(request, "add");
        if (privilegesToBeAdded != null) {
            for (String privilege : privilegesToBeAdded) {
                User.addClientLevelPrivilege(client, email, User.ClientLevelPrivilege.valueOf(privilege), login);
            }
        }

        Set<String> privilegesToBeRemoved = ServletUtils.getStringParamsAsSet(request, "remove");
        if (privilegesToBeRemoved != null) {
            for (String privilege : privilegesToBeRemoved) {
                User.deleteClientLevelPrivilege(client, email, User.ClientLevelPrivilege.valueOf(privilege), login);
            }
        }

        ServletUtils.setJson(response, new APIResponse().object(User.safeGet(client, email).toProp(client)).status(Status.SUCCESS));
    }

    private void updateGroupLevelPrivileges(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        String email = ServletUtils.getStrParam(request, "email");
        long groupId = Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "groupName")).toProp().groupId;

        Set<String> privilegesToBeAdded = ServletUtils.getStringParamsAsSet(request, "add");
        if (privilegesToBeAdded != null) {
            for (String privilege : privilegesToBeAdded) {
                User.addGroupLevelPrivilege(client, groupId, email, User.GroupLevelPrivilege.valueOf(privilege), login);
            }
        }

        Set<String> privilegesToBeRemoved = ServletUtils.getStringParamsAsSet(request, "remove");
        if (privilegesToBeRemoved != null) {
            for (String privilege : privilegesToBeRemoved) {
                User.deleteGroupLevelPrivilege(client, groupId, email, User.GroupLevelPrivilege.valueOf(privilege), login);
            }
        }

        ServletUtils.setJson(response, new APIResponse().object(User.safeGet(client, email).toProp(client)).status(Status.SUCCESS));
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {

            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "create":
                        create(client, login, request, response);
                        break;
                    case "linkMember":
                        linkMember(client, login, request, response);
                        break;
                    case "getAll":
                        getAll(client, login, request, response);
                        break;
                    case "getClientLevelPrivileges":
                        getClientLevelPrivileges(client, login, request, response);
                        break;
                    case "getGroupLevelPrivileges":
                        getGroupLevelPrivileges(client, login, request, response);
                        break;
                    case "updateClientLevelPrivileges":
                        updateClientLevelPrivileges(client, login, request, response);
                        break;
                    case "updateGroupLevelPrivileges":
                        updateGroupLevelPrivileges(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }
}
