package crmdna.client.isha;

import crmdna.api.servlet.RegistrationServlet;
import crmdna.api.servlet.ServletUtils;
import crmdna.common.Utils.Pair;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.registration.Registration;
import crmdna.registration.Registration.RegistrationStatus;
import crmdna.registration.RegistrationEntity;
import crmdna.registration.RegistrationProp;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.logging.Logger;

import static crmdna.common.OfyService.ofy;

public class IshaRegistrationServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;
    private final static Logger LOGGER = Logger.getLogger(IshaRegistrationServlet.class.getName());

    private void volunteerRegistration(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        /* Register as a normal program */
        RegistrationProp prop = RegistrationServlet.registerForProgram(request, response);
        if ((prop != null) && (prop.status == RegistrationStatus.PAYMENT_AUTHORIZATION_PENDING)) {
            RegistrationEntity entity = Registration.safeGet("isha", prop.registrationId);

            /*
             After payment has been initiated, save the query params in the registration entity
             Once payment is completed, the flow with continue with posting to the google sheet (refer: handlePostRegistration)
            */
            List<Pair<String>> params = new ArrayList<>();
            Enumeration paramNames = request.getParameterNames();
            while (paramNames.hasMoreElements()) {
                String paramName = paramNames.nextElement().toString();
                params.add(new Pair<>(paramName, request.getParameter(paramName)));
            }

            entity.setTxReference(ServletUtils.getQuery(params));
            ofy("isha").save().entity(entity).now();
        }
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {
            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "volunteerRegistration":
                        volunteerRegistration(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }

}
