package crmdna.client.isha.ieo;

import com.google.appengine.api.utils.SystemProperty;
import crmdna.common.api.APIResponse;
import crmdna.common.api.APIResponse.Status;
import crmdna.common.api.APIUtils;
import crmdna.common.api.RequestInfo;
import crmdna.api.servlet.ServletUtils;
import crmdna.registration.RegistrationProp;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

public class IshaIEOServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    private void syncSMMRegistrations(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        List<RegistrationProp> props = IshaSMM.getRegistrations(
            ServletUtils.getLongParam(request, "programId"), login);

        ServletUtils.setJson(response, new APIResponse().status(Status.SUCCESS)
            .message("Synced " + props.size() + " registrations"));
    }

    private void log(String client, String login, HttpServletRequest request,
        HttpServletResponse response) throws Exception {

        Logger LOGGER = Logger.getLogger("DebugHttp");

        Enumeration e = request.getParameterNames();
        while (e.hasMoreElements()) {
            String name = (String) e.nextElement();
            LOGGER.info(name + ": " + request.getParameter(name) + "\n");
        }

        if (request.getParameter("fName").contains("E")) {
            ServletUtils.setJson(response,
                "{\"Status\":\"1\","
                    + "\"Msg\":\"Error: 'Payment Failed:'"
                    + "status=1&"
                    + "fCode=10527&"
                    + "fMesg=This transaction cannot be processed. "
                    + "Please enter a valid credit card number and type.\"}");
        } else {
            ServletUtils.setJson(response,
                "{\"Status\":\"Success\","
                    + "\"Confirm\":\"Success\","
                    + "\"Msg\":\"User Created successfully\","
                    + "\"aUid\":9625,"
                    + "\"aPid\":\"1240893417610005\""
                    + "}");
        }

        /*
            {"Status":"Success","Confirm":"Success","Msg":"User Created successfully",
             "aUid":9625,"aPid":"1240893417610005"}
            {"Status":"1","Msg":"Error: 'Payment Failed:'status=1&fCode=10527&
             fMesg=This transaction cannot be processed. Please enter a valid credit card number and type." }
        */
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {

            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "syncApacProfiles":
                        if (SystemProperty.applicationId.get().equalsIgnoreCase("ishacrmserver")) {
                            IshaIEO.syncApacProfiles();
                        }
                        break;
                    case "log":
                        log(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

        String action = request.getParameter("action");
        if (action == null) {
            ServletUtils.setJson(response, new APIResponse().status(Status.ERROR_RESOURCE_NOT_FOUND));
        } else {
            String client = request.getParameter("client");
            if (client == null)
                client = "isha";

            String login = ServletUtils.getLogin(request);

            try {
                switch (action) {
                    case "syncSMMRegistrations":
                        syncSMMRegistrations(client, login, request, response);
                        break;
                    case "log":
                        log(client, login, request, response);
                        break;
                    default:
                        ServletUtils.setJson(response,
                            new APIResponse().status(Status.ERROR_RESOURCE_INCORRECT));
                }
            } catch (Exception ex) {
                ServletUtils.setJson(response, APIUtils.toAPIResponse(ex, true,
                    new RequestInfo().client(client).req(request).login(login)));
            }
        }
    }

}
