package crmdna.interaction;

import com.googlecode.objectify.annotation.*;
import com.googlecode.objectify.condition.IfNotNull;
import crmdna.calling.Campaign;
import crmdna.user.User;
import crmdna.user.UserProp;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

@Entity
@Cache
public class InteractionEntity {
    @Id long interactionId;

    @Serialize(zip = true) Map<Long, String> subInteractions = new HashMap<>();

    // key is nano seconds, value is sub interaction content
    // map is always descending sorted by key

    @Index long ms;

    @Index String interactionType;

    @Index long userId;

    @Index long memberId;

    @Index(IfNotNull.class) Long campaignId;
    @Index(IfNotNull.class) Long campaignProgramId;

    @Index boolean isTask;

    @Index Interaction.Progress progress;

    @Index Interaction.Response response;

    public InteractionProp toProp(String client) {
        InteractionProp interactionProp = new InteractionProp();

        interactionProp.interactionId = interactionId;
        interactionProp.timestamp = ms;

        for (Long subInteractionId : subInteractions.keySet()) {
            SubInteractionProp subInteractionProp = new SubInteractionProp();
            subInteractionProp.subInteractionId = subInteractionId;
            subInteractionProp.content = subInteractions.get(subInteractionId);
            subInteractionProp.timestamp = subInteractionId / 1000;
            interactionProp.subInteractionProps.add(subInteractionProp);
        }
        Collections.sort(interactionProp.subInteractionProps);

        interactionProp.interactionType = interactionType;

        interactionProp.userId = userId;
        UserProp userProp = User.safeGetById(client, userId).toProp(client);
        interactionProp.userFirstName = userProp.firstName;
        interactionProp.userLastName = userProp.lastName;

        interactionProp.memberId = memberId;
        interactionProp.campaignId = campaignId;
        if (campaignId != null) {
            interactionProp.campaignName = Campaign.safeGet(client, campaignId).toProp().shortName;
        }

        interactionProp.progress = progress;
        interactionProp.response = response;

        return interactionProp;
    }

    public Long getCampaignId() {
        return campaignId;
    }
}
