package crmdna.mail2;

import crmdna.common.Utils;

import java.util.*;

import static crmdna.common.AssertUtils.ensure;
import static crmdna.common.AssertUtils.ensureNotNull;

public class MailMap {
    public enum MergeVarID {
        MAIL_ID,
        FIRST_NAME,
        LAST_NAME,
        EMAIL,
        MOBILE_PHONE,
        HOME_PHONE,
        OFFICE_PHONE,
        VERIFICATION_CODE,
        PASSWORD,
        SUBSCRIPTION_TYPE,
        AMOUNT,
        AMOUNT_IN_WORDS,
        VALIDITY,
        PROGRAM_NAME,
        REGISTRATION_ID,
        DATE,
        DATES,
        VENUE,
        ADDRESS,
        MAP_LINK,
        SESSIONS,
        TRANSACTION_ID,
        INVOICE_NUMBER,
        PURPOSE,
        VAR1,
        VAR2,
        VAR3,
        VAR4,
        VAR5,
        VAR6,

        //scheduled email
        MAIL_SUBJECT,
        MAIL_DISPLAY_NAME,
        NUM_RECIEPIENTS,
        LIST,
        GROUP,
        MAIL_STATS_URL,
        MAIL_CONTENT_ID,
        SCHEDULED_MAIL_ID,
        SCHEDULED_TIME,
        FAILURE_REASON
    }

    private Map<String, Map<MergeVarID, String>> map = new HashMap<>();
    private Map<MergeVarID, String> globalMap = new HashMap<>();
    private Map<MergeVarID, Object> globalObjectMap = new HashMap<>();

    public void add(String email) {
        ensure(Utils.isValidEmailAddress(email), "Email is invalid");
        map.put(email.toLowerCase(), new HashMap<MergeVarID, String>());
    }

    public void add(String email, String firstName, String lastName) {
        email = email.toLowerCase();
        add(email);
        setMergeVar(email, MergeVarID.FIRST_NAME, firstName);
        setMergeVar(email, MergeVarID.LAST_NAME, lastName);
    }

    public void setMergeVar(String email, MergeVarID id, String value) {
        email = email.toLowerCase();
        Map<MergeVarID, String> _map = map.get(email);
        if (_map == null) {
            _map = new HashMap<>();
        }
        _map.put(id, value);
        map.put(email, _map);
    }

    public void setGlobalMergeVar(MergeVarID id, String value) {
        globalMap.put(id, value);
    }

    public void setGlobalMergeVar(MergeVarID id, Object value) {
        globalObjectMap.put(id, value);
    }

    public Set<String> getEmails() {
        return Collections.unmodifiableSet(map.keySet());
    }

    public long getMailId(String email) {
        return Long.parseLong(map.get(email).get(MergeVarID.MAIL_ID));
    }

    public String getMergeVar(MergeVarID id, String email) {

        ensure(map.containsKey(email), "Email [" + email + "] missing in MailMap");

        return this.map.get(email).get(id);
    }

    public String getGlobalMergeVar(MergeVarID id) {
        return globalMap.get(id);
    }

    public Object getGlobalMergeVarObject(MergeVarID id) {
        return globalObjectMap.get(id);
    }

    public int size() {
        return map.size();
    }

    public boolean isEmpty() {
        return map.isEmpty();
    }

    public void delete(String email) {
        ensureNotNull(email, "email is null");
        map.remove(email);
    }

    public Map<String, Map<MergeVarID, String>> getMap() {
        return map;
    }

    public void validateMergeVars() {
        Set<MailMap.MergeVarID> prevSet = null;
        String prevEmail = "";
        for (Map.Entry<String, Map<MailMap.MergeVarID, String>> entry : map.entrySet()) {
            Map<MailMap.MergeVarID, String> mergeVarMap = entry.getValue();
            if (prevSet != null) {
                ensure(prevSet.equals(mergeVarMap.keySet()),
                    "Number of mergeVars differ between entries "
                        + "(" + prevEmail + ":" + Arrays.toString(prevSet.toArray()) + ") <-> "
                        + "(" + entry.getKey() + ":" + Arrays.toString(mergeVarMap.keySet().toArray()) + ")");
            }
            prevSet = mergeVarMap.keySet();
            prevEmail = entry.getKey();
        }
    }

    public void populateMailIds() {

        Long mailId = (new Date().getTime()) * 1000000;
        for (Map.Entry<String, Map<MergeVarID, String>> entry : map.entrySet())
        {
            Map<MergeVarID, String> mergeVarMap = entry.getValue();
            mergeVarMap.put(MergeVarID.MAIL_ID, (mailId++).toString());
        }
    }
}
