package crmdna.mail2;

import com.googlecode.objectify.annotation.Entity;
import com.googlecode.objectify.annotation.Id;
import com.googlecode.objectify.annotation.Index;
import com.googlecode.objectify.annotation.Serialize;
import com.googlecode.objectify.condition.IfNotNull;
import com.googlecode.objectify.condition.IfTrue;
import crmdna.common.StackTraceElementProp;

import java.util.Date;
import java.util.List;

@Entity
public class MailScheduleEntity {
    @Id
    long mailScheduleId;

    @Index
    long mailContentId;

    @Index
    long listId;

    @Index(IfNotNull.class)
    Long programId;

    String defaultFirstName;
    String defaultLastName;

    @Index
    String userEmail;

    @Index
    long groupId;

    @Index
    String senderEmail;

    @Index
    long scheduledTimeMs;

    @Index
    Long maxTimeMs;

    @Index(IfTrue.class)
    boolean processedAfterMaxTime;

    @Index
    boolean enabled;

    @Index
    boolean processed;

    Long processTimeMs;

    @Index
    Boolean sendSuccess;

    int numRecipients;

    MailSchedule.FailureCode failureCode;
    String failureMessage;

    @Serialize(zip = true)
    List<StackTraceElementProp> stackTraceElementProps;


    public MailScheduleProp toProp() {
        MailScheduleProp mailScheduleProp = new MailScheduleProp();

        mailScheduleProp.mailScheduleId = mailScheduleId;
        mailScheduleProp.mailContentId = mailContentId;
        mailScheduleProp.userEmail = userEmail;
        mailScheduleProp.groupId = groupId;
        mailScheduleProp.listId = listId;
        mailScheduleProp.scheduledTime = new Date(scheduledTimeMs);
        mailScheduleProp.maxTime = new Date(maxTimeMs);
        mailScheduleProp.sendAttempted = processed;
        if (processTimeMs != null) {
            mailScheduleProp.processTime = new Date(processTimeMs);
        }
        mailScheduleProp.processed = processed;
        mailScheduleProp.enabled = enabled;
        mailScheduleProp.sendSuccess = sendSuccess;
        mailScheduleProp.numRecipients = numRecipients;
        mailScheduleProp.failureCode = failureCode;
        mailScheduleProp.failureMessage = failureMessage;
        mailScheduleProp.stackTraceElementProps = stackTraceElementProps;

        return mailScheduleProp;
    }
}
