package crmdna.member;

import com.google.appengine.api.taskqueue.QueueFactory;
import com.google.appengine.api.taskqueue.TaskOptions;
import com.google.gson.Gson;
import crmdna.api.servlet.ServletUtils;
import crmdna.common.DateUtils;
import crmdna.group.Group;
import crmdna.objectstore.ObjectStore;
import crmdna.program.Program;
import crmdna.program.ProgramProp;
import crmdna.program.ProgramQueryCondition;
import crmdna.programtype.ProgramType;
import crmdna.programtype.ProgramTypeProp;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashSet;
import java.util.List;

import static crmdna.common.AssertUtils.ensureNotNull;

public class MemberUtils {
    public static void queryAsync(MemberQueryCondition condition, String login, Integer partNumber,
        Integer totalParts, Integer resultSize) {

        ensureNotNull(condition.maxResultSize, "Max Result Size should not be null");
        if ((resultSize != null) && (totalParts == null) && (partNumber == null)) {
            totalParts = (int) Math.ceil(((double) resultSize) / condition.maxResultSize);
            partNumber = 1;
        }
        com.google.appengine.api.taskqueue.Queue queue = QueueFactory.getDefaultQueue();
        long tempAccessId = ObjectStore
            .put(condition.client, "TempAccessID", 30, ObjectStore.TimeUnit.SECONDS);
        queue.add(TaskOptions.Builder.withUrl("/member")
            .param("action", "sendReportAsEmail")
            .param("qc", new Gson().toJson(condition))
            .param("email", login)
            .param("emailAttachmentName", "MemberReport-" + DateUtils.toISOString(new Date(), false))
            .param("accessId", tempAccessId + "").param("partNumber", partNumber + "")
            .param("totalParts", totalParts + ""));
    }

    public static MemberQueryCondition getQueryCondition(String client, HttpServletRequest request) {

        Integer maxResultSize = ServletUtils.getIntParam(request, "maxResultSize");

        MemberQueryCondition qc = new MemberQueryCondition(client, maxResultSize);
        qc.programTypeIds = ServletUtils.getLongParamsAsSet(request, "programTypeId");
        if (qc.programTypeIds == null) {
            String programType = ServletUtils.getStrParam(request, "programType");
            if (programType != null) {
                qc.programTypeIds = new HashSet<>();
                ProgramTypeProp programTypeProp = ProgramType.safeGetByIdOrName(client, programType).toProp(client);
                qc.programTypeIds.add(programTypeProp.programTypeId);
            }
        }

        qc.programIds = ServletUtils.getLongParamsAsSet(request, "programId");
        if (qc.programIds == null) {
            ProgramQueryCondition pqc = new ProgramQueryCondition();

            pqc.startYYYYMMDD = ServletUtils.getIntParam(request, "startYYYYMMDD");
            pqc.endYYYYMMDD = ServletUtils.getIntParam(request, "endYYYYMMDD");
            if ((qc.programTypeIds != null) || (pqc.startYYYYMMDD != null) || (pqc.endYYYYMMDD != null)) {
                pqc.programTypeIds = qc.programTypeIds;

                List<ProgramProp> programProps = Program.query(client, pqc);
                qc.programIds = new HashSet<>();
                for (ProgramProp programProp : programProps) {
                    qc.programIds.add(programProp.programId);
                }
            }
        }

        qc.groupIds.add(Group.safeGetByIdOrName(client, ServletUtils.getStrParam(request, "group")).toProp().groupId);
        qc.practiceIds = ServletUtils.getLongParamsAsSet(request, "practiceId");
        qc.occupations = ServletUtils.getLongParamsAsSet(request, "occupation");
        qc.listIds = ServletUtils.getLongParamsAsSet(request, "listId");
        qc.queryNoShows = ServletUtils.getBoolParam(request, "queryNoShows");

        return qc;
    }
}
