package crmdna.registration;

import com.googlecode.objectify.annotation.Cache;
import com.googlecode.objectify.annotation.Entity;
import com.googlecode.objectify.annotation.Id;
import com.googlecode.objectify.annotation.Index;
import crmdna.common.Utils.PaypalErrorType;
import crmdna.common.contact.Contact.Gender;
import crmdna.registration.Registration.RegistrationStatus;

import java.util.*;

import static crmdna.common.OfyService.ofy;

@Entity
@Cache
public class RegistrationEntity {
    @Id
    long registrationId;
    String firstName;
    String lastName;
    String nickName;

    @Index
    String email;

    Gender gender;
    String mobilePhone;
    String homePhone;
    String officePhone;
    String homeAddress;
    String officeAddress;
    String country;
    String city;
    String postalCode;

    @Index
    long memberId;
    @Index
    long programId;

    String paymentUrl;
    String redirectUrl;

    String marketingChannel;

    String successCallbackUrl;
    String errorCallbackUrl;

    // paypal fields
    String amount;
    String ccy;
    boolean isPaymentPending;
    String pendingReason;
    PaypalErrorType paypalErrorType;
    String L_SEVERITYCODE0;
    String L_ERRORCODE0;
    String L_SHORTMESSAGE0;
    String L_LONGMESSAGE0;

    @Index
    String transactionId;
    @Index
    Set<String> qsTags = new HashSet<>();

    String txReference;

    @Index
    private RegistrationStatus status;
    // track different states
    private List<Date> stateChangeTimestamps = new ArrayList<>();
    private List<RegistrationStatus> statuses = new ArrayList<>();

    RegistrationStatus getStatus() {
        return status;
    }

    void recordStateChange(RegistrationStatus rs) {
        status = rs;
        stateChangeTimestamps.add(new Date());
        statuses.add(rs);
    }

    public void overrideStateChange(int index, RegistrationStatus rs, Date newDate) {
        if (index < statuses.size()) {
            statuses.set(index, rs);
            stateChangeTimestamps.set(index, newDate);
        } else {
            statuses.add(rs);
            stateChangeTimestamps.add(newDate);
        }
    }

    public void setTxReference(String txReference) {
        this.txReference = txReference;
    }

    public RegistrationEntity update (RegistrationProp prop) {
        firstName = prop.firstName;
        lastName = prop.lastName;
        email = prop.email;
        gender = prop.gender;
        mobilePhone = prop.mobilePhone;
        homePhone = prop.homePhone;
        country = prop.country;
        city = prop.city;
        postalCode = prop.postalCode;
        return this;
    }

    public RegistrationProp toProp() {
        RegistrationProp prop = new RegistrationProp();

        prop.registrationId = registrationId;
        prop.firstName = firstName;
        prop.lastName = lastName;
        prop.nickName = nickName;
        prop.gender = gender;
        prop.email = email;
        prop.mobilePhone = mobilePhone;
        prop.homePhone = homePhone;
        prop.officePhone = officePhone;
        prop.homeAddress = homeAddress;
        prop.officeAddress = officeAddress;
        prop.country = country;
        prop.city = city;
        prop.postalCode = postalCode;

        prop.memberId = memberId;
        prop.programId = programId;

        prop.paymentUrl = paymentUrl;
        prop.redirectUrl = redirectUrl;

        prop.successCallbackUrl = successCallbackUrl;
        prop.errorCallbackUrl = errorCallbackUrl;

        prop.amount = amount;
        prop.ccy = ccy;
        prop.transactionId = transactionId;
        prop.isPaymentPending = isPaymentPending;
        prop.pendingReason = pendingReason;
        prop.paypalErrorType = paypalErrorType;
        prop.L_SEVERITYCODE0 = L_SEVERITYCODE0;
        prop.L_ERRORCODE0 = L_ERRORCODE0;
        prop.L_SHORTMESSAGE0 = L_SHORTMESSAGE0;
        prop.L_LONGMESSAGE0 = L_LONGMESSAGE0;

        prop.txReference = txReference;

        for (int i = 0; i < statuses.size(); i++) {
            RegistrationStatusChange rsc = new RegistrationStatusChange();
            rsc.newStatus = statuses.get(i);
            rsc.timestamp = stateChangeTimestamps.get(i);

            if (rsc.newStatus == RegistrationStatus.REGISTRATION_COMPLETE) {
                prop.registeredMs = rsc.timestamp.getTime();
            }

            prop.changes.add(rsc);
        }

        prop.status = status;

        return prop;
    }
}
