package crmdna.user;

import crmdna.client.Client;
import crmdna.common.Utils;
import crmdna.user.User.GroupLevelPrivilege;

import java.util.*;

import static crmdna.common.AssertUtils.ensureNotNull;

public class UserHelper {
    public static Set<Long> getGroupIdsWithPrivilage(String client, String email,
                                                     GroupLevelPrivilege privilege) {

        Client.ensureValid(client);
        UserEntity userEntity = User.safeGet(client, email);
        ensureNotNull(privilege, "privilege is null");

        Set<Long> groupIds = new HashSet<>();
        for (String rawPrivilege : userEntity.privileges) {
            String[] split = rawPrivilege.split("\\|\\|");

            if (split.length != 3)
                continue;

            String resourceType = split[0];
            String resource = split[1];
            String action = split[2];

            if (resourceType.equalsIgnoreCase("GROUP") && action.equalsIgnoreCase(privilege.toString())) {
                if (Utils.canParseAsLong(resource))
                    groupIds.add(Utils.safeParseAsLong(resource));
            }
        }

        return groupIds;
    }

    public static Map<Long, String> getAllUserIdVsEmail(String client, String login) {
        //sathya todo: add test case
        Client.ensureValid(client);

        List<UserProp> props = User.getAll(client, login);

        Map<Long, String> userIdVsEmail = new HashMap<>();
        for (UserProp prop : props) {
            if (prop.email != null) {
                userIdVsEmail.put(prop.userId, prop.email);
            }
        }

        return userIdVsEmail;
    }
}
