/*
 * Copyright (C) 2010 Pete Reisinger <p.reisinger@gmail.com>.
 * 
 * This program is free software: you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program. If
 * not, see <http://www.gnu.org/licenses/>.
 */

package paypalnvp.request;

import java.util.HashMap;
import java.util.Map;

/**
 * Obtain the available balance for a PayPal account.
 *
 * @author Pete Reisinger
 *         <p.reisinger@gmail.com>
 */
@SuppressWarnings("serial")
public final class GetBalance implements Request {

    /**
     * Method value of this request
     */
    private static final String METHOD_NAME = "GetBalance";

    /**
     * map that holds name value pair request values
     */
    private final Map<String, String> nvpRequest;

    /**
     * map that holds name value pair response values
     */
    private Map<String, String> nvpResponse;

    public GetBalance() {

        nvpResponse = new HashMap<String, String>();
        nvpRequest = new HashMap<String, String>();
        nvpRequest.put("METHOD", METHOD_NAME);
    }

    /**
     * Whether to return all currencies.
     *
     * @param allCurrencies true - Return the balance for each currency holding false - Return only
     *                      the balance for the primary currency holding
     */
    public void setAllCurrencies(boolean allCurrencies) {

        String all = (allCurrencies) ? "1" : "0";
        nvpRequest.put("RETURNALLCURRENCIES", all);
    }

    public Map<String, String> getNVPRequest() {
        return new HashMap<String, String>(nvpRequest);
    }

    public Map<String, String> getNVPResponse() {
        return new HashMap<String, String>(nvpResponse);
    }

    public void setNVPResponse(Map<String, String> nvpResponse) {
        this.nvpResponse = new HashMap<String, String>(nvpResponse);
    }

    @Override
    public String toString() {

        StringBuffer str = new StringBuffer("instance of GetBalance");
        str.append("class with the vlues: nvpRequest - ");
        str.append(nvpRequest.toString());
        str.append("; nvpResponse - ");
        str.append(nvpResponse.toString());

        return str.toString();
    }
}
