package crmdna.client.isha;

import com.google.appengine.tools.development.testing.LocalDatastoreServiceTestConfig;
import com.google.appengine.tools.development.testing.LocalServiceTestHelper;
import com.googlecode.objectify.ObjectifyFilter;
import crmdna.client.Client;
import crmdna.client.isha.ieo.IshaIEO;
import crmdna.client.isha.ieo.IshaSMM;
import crmdna.common.Utils;
import crmdna.group.Group;
import crmdna.group.Group.GroupProp;
import crmdna.practice.Practice;
import crmdna.practice.Practice.PracticeProp;
import crmdna.program.Program;
import crmdna.program.ProgramProp;
import crmdna.programtype.ProgramType;
import crmdna.programtype.ProgramTypeProp;
import crmdna.teacher.Teacher;
import crmdna.teacher.Teacher.TeacherProp;
import crmdna.user.User;
import crmdna.venue.Venue;
import crmdna.venue.Venue.VenueProp;
import org.junit.Before;
import org.junit.Test;

import java.util.HashSet;
import java.util.Set;

public class IshaIEOTest {
    private final LocalServiceTestHelper datastoreHelper = new LocalServiceTestHelper(
            new LocalDatastoreServiceTestConfig().setApplyAllHighRepJobPolicy());

    GroupProp sgp;
    ProgramProp smm;
    long smmProgramId = 160;

    @Before
    public void setUp() {
        datastoreHelper.setUp();
        ObjectifyFilter.complete();

        String client = "isha";

        Client.create(client);
        sgp = Group.create(client, "Isha Singapore", User.SUPER_USER);
        Group.create(client, "Isha Hong Kong", User.SUPER_USER);
        Group.create(client, "Isha Australia", User.SUPER_USER);
        Group.create(client, "Isha Malaysia", User.SUPER_USER);
        Group.create(client, "Isha UAE", User.SUPER_USER);
        Group.create(client, "Isha Oman", User.SUPER_USER);
        Group.create(client, "Isha Kuwait", User.SUPER_USER);
        Group.create(client, "Isha Bahrain", User.SUPER_USER);
        Group.create(client, "Isha Qatar", User.SUPER_USER);

        PracticeProp shambhavi = Practice.create(client, "Shambhavi", User.SUPER_USER);
        Set<Long> practiceIds = new HashSet<>();
        practiceIds.add(shambhavi.practiceId);
        ProgramTypeProp smmType =
            ProgramType.create(client, "SMM 2Day", practiceIds, false, User.SUPER_USER);
        VenueProp giis = Venue.create(client, "GIIS", "GIIS", "GIIS", "", sgp.groupId, User.SUPER_USER);
        TeacherProp nidhi =
            Teacher.create(client, "Nidhi", "Jain", "tina@ishafoundation.org", sgp.groupId,
                User.SUPER_USER);
        smm = Program.create("isha", 1, smmType.programTypeId, giis.venueId,
            nidhi.teacherId, 20151229, 20151229, 1, null, 200.0, Utils.Currency.SGD, User
                .SUPER_USER);
        IshaSMM.createProgramMap(181, smm.programId, sgp.groupId, "General", User.SUPER_USER);
        IshaSMM.createProgramMap(185, smm.programId, sgp.groupId, "Preferred", User.SUPER_USER);
    }

    @Test
    public void readSMMRegistrations() {
        try {
            IshaSMM.readRegistrations(smmProgramId);
        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }

    @Test
    public void syncSMMRegistrations() {
        try {
            IshaIEO.syncApacProfiles();
            ObjectifyFilter.complete();
            IshaSMM.getRegistrations(smm.programId, User.SUPER_USER);
            IshaSMM.getRegistrations(smm.programId, User.SUPER_USER);
            IshaSMM.update(smm, User.SUPER_USER);

        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }

    @Test
    public void syncApacProfiles() {
        try {
            IshaIEO.syncApacProfiles();
        } catch (Exception e) {
            System.out.println(e.toString());
        }
    }

}
